#!/usr/bin/env python
# 
# $Id: Nielsen2011New.py,v 1.10 2011/03/16 13:41:36 fn Exp $

import csv
import re
import numpy as np
from nltk.stem.wordnet import WordNetLemmatizer
from nltk import sent_tokenize
import pylab
from scipy.stats.stats import  kendalltau, spearmanr
import simplejson

# This variable determines the data set
mislove = 2

# Filenames
filebase = '/home/fnielsen/'
filename_afinn = filebase + 'fnielsen/data/Nielsen2009Responsible_emotion.csv'
filename_afinn96 = 'AFINN-96.txt'
filename_mislove1 = "results.txt"
filename_mislove2a = "turk.txt"
filename_mislove2b = "turk-results.txt"
filename_anew = filebase + 'data/ANEW.TXT'
filename_gi = filebase + "data/inqtabs.txt"
filename_of = filebase + "data/subjclueslen1-HLTEMNLP05.tff"

# Word splitter pattern
pattern_split = re.compile(r"\W+")


afinn = dict(map(lambda (k,v): (k,int(v)), 
                 [ line.split('\t') for line in open(filename_afinn) ]))


afinn96 = dict(map(lambda (k,v): (k,int(v)), 
                 [ line.split('\t') for line in open(filename_afinn96) ]))


# ANEW
anew = dict(map(lambda l: (l[0], float(l[2]) - 5) , 
                [ re.split('\s+', line) for line in open(filename_anew).readlines()[41:1075] ]))


# OpinionFinder
of = {}
for line in open(filename_of).readlines():
    elements = re.split('(\s|\=)', line)
    if elements[22] == 'positive':
        of[elements[10]] = +1
    elif elements[22] == 'negative':
        of[elements[10]] = -1


# General inquirer
csv_reader = csv.reader(open(filename_gi), delimiter='\t')
header = []
gi = {}
previousword = []
previousvalence = []
for row in csv_reader:
    if not header:
        header = row
    elif len(row) > 2:
        word = re.search("\w+", row[0].lower()).group()
        if row[2] == "Positiv":
            valence = 1
        elif row[3] == "Negativ":
            valence = -1
        else:
            valence = 0
        if re.search("#", row[0].lower()):
            if previousword == word:
                if previousvalence == []:
                    previousvalence = valence
                elif previousvalence != valence:
                    previousvalence = 0 
            else: 
                if previousvalence:
                    gi[previousword] = previousvalence
                previousword = word
                previousvalence = []
        elif valence:
            gi[word] = valence



# Lemmatizer for WordNet
lemmatizer = WordNetLemmatizer()



def words2anewsentiment(words):
    """
    Convert words to sentiment based on ANEW via WordNet Lemmatizer
    """
    sentiment = 0 
    for word in words:
        if word in anew:
            sentiment += anew[word]
            continue
        lword = lemmatizer.lemmatize(word)
        if lword in anew:
            sentiment += anew[lword]
            continue
        lword = lemmatizer.lemmatize(word, pos='v')
        if lword in anew:
            sentiment += anew[lword]
    return sentiment/len(words)  


def extremevalence(valences):
    """
    Return the most extreme valence. If extremes have different sign then
    zero is returned
    """
    imax = np.argsort(np.abs(valences))[-1]
    extremes = filter(lambda v: abs(v) == abs(valences[imax]), valences)
    extremes_samesign = filter(lambda v: v == valences[imax], valences)
    if extremes == extremes_samesign:
        return valences[imax]
    else:
        return 0

def corrmatrix2latex(C, columns=None): 
    """
    s = corrmatrix2latex(C)
    print(s)
    """
    s = '\n\\begin{tabular}{' + ('r'*(C.shape[0]-1)) + '}\n'
    if columns:
        s += " & ".join(columns[1:]) + " \\\\\n\\hline\n"
    for n in range(C.shape[0]):
        row = []
        for m in range(1, C.shape[1]):
            if m > n: 
                row.append("%.3f" % C[n,m])
            else:
                row.append("     ")
        s += " & ".join(row) + ' \\\\\n'
    s += '\\end{tabular}\n'
    return s


def spearmanmatrix(data):
    """
    Spearman r rank correlation matrix
    """
    C = np.zeros((data.shape[1], data.shape[1]))
    for n in range(data.shape[1]):
        for m in range(data.shape[1]):
            C[n,m] = spearmanr(data[:,n], data[:,m])[0]
    return C


def kendallmatrix(data):
    """
    Kendall tau rank correlation matrix
    """
    C = np.zeros((data.shape[1], data.shape[1]))
    for n in range(data.shape[1]):
        for m in range(data.shape[1]):
            C[n,m] = kendalltau(data[:,n], data[:,m])[0]
    return C



# Read Mislove CSV Twitter data: 'tweets' an array of dictionaries
if mislove == 1:
    csv_reader = csv.reader(open(filename_mislove1), delimiter='\t')
    header = []
    tweets = []
    for row in csv_reader:
        if not header:
            header = row
        else:
            tweets.append({'id': row[0], 
                           'quant': int(row[1]), 
                           'score_our': float(row[2]),
                           'score_mean': float(row[3]),
                           'score_std': float(row[4]),
                           'text': row[5], 
                           'scores': map(int, row[6:])})
elif mislove == 2:
    if True: 
        tweets = simplejson.load(open('tweets_with_sentistrength.json'))
    else:
        csv_reader = csv.reader(open(filename_mislove2a), delimiter='\t')
        tweets = []
        for row in csv_reader:
            tweets.append({'id': row[2], 
                           'score_mislove2': float(row[1]),
                           'text': row[3]})
        csv_reader = csv.reader(open(filename_mislove2b), delimiter='\t')
        tweets_dict = {}
        header = []
        for row in csv_reader:
            if not header:
                header = row
            else:
                tweets_dict[row[0]] = {'id': row[0], 
                                       'score_mislove': float(row[1]), 
                                       'score_amt_wrong': float(row[2]),
                                       'score_amt': np.mean(map(int, re.split("\s+", " ".join(row[4:]).strip())))
                                       }
        for n in range(len(tweets)):
            tweets[n]['score_mislove'] = tweets_dict[tweets[n]['id']]['score_mislove']
            tweets[n]['score_amt_wrong'] = tweets_dict[tweets[n]['id']]['score_amt_wrong']
            tweets[n]['score_amt'] = tweets_dict[tweets[n]['id']]['score_amt']


# Add sentiments to 'tweets'
for n in range(len(tweets)):
    words = pattern_split.split(tweets[n]['text'].lower())
    afinn_sentiments = map(lambda word: afinn.get(word, 0), words)
    afinn_sentiment = float(sum(afinn_sentiments))/len(afinn_sentiments)
    afinn96_sentiments = map(lambda word: afinn96.get(word, 0), words)
    afinn96_sentiment = float(sum(afinn96_sentiments))/len(afinn96_sentiments)
    anew_sentiments = map(lambda word: anew.get(word, 0), words)
    anew_sentiment = float(sum(anew_sentiments))/len(anew_sentiments)
    gi_sentiments = map(lambda word: gi.get(word, 0), words)
    gi_sentiment = float(sum(gi_sentiments))/len(gi_sentiments)
    of_sentiments = map(lambda word: of.get(word, 0), words)
    of_sentiment = float(sum(of_sentiments))/len(of_sentiments)
    tweets[n]['sentiment_afinn96'] = afinn96_sentiment
    tweets[n]['sentiment_afinn'] = afinn_sentiment
    tweets[n]['sentiment_afinn_quant'] = np.sign(afinn_sentiment)
    tweets[n]['sentiment_afinn_sum'] = sum(afinn_sentiments)
    nonzeros = len(filter(lambda nonzero: nonzero, afinn_sentiments))
    if not nonzeros: nonzeros = 1
    tweets[n]['sentiment_afinn_nonzero'] = sum(afinn_sentiments)/nonzeros
    tweets[n]['sentiment_afinn_extreme'] = extremevalence(afinn_sentiments)
    tweets[n]['sentiment_anew_raw'] = anew_sentiment
    tweets[n]['sentiment_anew_lemmatize'] = words2anewsentiment(words)
    tweets[n]['sentiment_anew_raw_sum'] = sum(anew_sentiments)
    nonzeros = len(filter(lambda nonzero: nonzero, anew_sentiments))
    if not nonzeros: nonzeros = 1
    tweets[n]['sentiment_anew_raw_nonzeros'] = sum(anew_sentiments)/nonzeros
    tweets[n]['sentiment_gi_raw'] = gi_sentiment
    tweets[n]['sentiment_of_raw'] = of_sentiment


# Numpy matrix
if mislove == 1:
    columns = ["AMT", 'AFINN', 'AFINN(q)', 'AFINN(s)', 'AFINN(a)',
               'ANEW(r)', 'ANEW(l)', 'ANEW(rs)', 'ANEW(a)']
    sentiments = np.matrix([ [t['score_mean'],
                              t['sentiment_afinn'], 
                              t['sentiment_afinn_quant'],
                              t['sentiment_afinn_sum'],
                              t['sentiment_afinn_nonzero'],
                              t['sentiment_anew_raw'], 
                              t['sentiment_anew_lemmatize'],
                              t['sentiment_anew_raw_sum'],
                              t['sentiment_anew_raw_nonzeros']] for t in tweets ])
elif mislove == 2:
    columns = ["AMT",  
               'AFINN', 'AFINN(q)', 'AFINN(s)', 'AFINN(a)', 'AFINN(x)', 
               'ANEW(r)', 'ANEW(l)', 'ANEW(rs)', 'ANEW(a)', 
               "GI", "OF", "SS"]
    sentiments = np.matrix([ [t['score_amt'],
                              t['sentiment_afinn'], 
                              t['sentiment_afinn_quant'],
                              t['sentiment_afinn_sum'],
                              t['sentiment_afinn_nonzero'],
                              t['sentiment_afinn_extreme'],
                              t['sentiment_anew_raw'], 
                              t['sentiment_anew_lemmatize'],
                              t['sentiment_anew_raw_sum'],
                              t['sentiment_anew_raw_nonzeros'],
                              t['sentiment_gi_raw'],
                              t['sentiment_of_raw'],
                              t['sentistrength']] for t in tweets ])


x = np.asarray(sentiments[:,1]).flatten()
y = np.asarray(sentiments[:,0]).flatten()
pylab.plot(x, y, '.')
pylab.xlabel('My list')
pylab.ylabel('Amazon Mechanical Turk')
pylab.text(0.1, 2, "Pearson correlation = %.3f" % np.corrcoef(x, y)[1,0])
pylab.text(0.1, 1.6, "Spearman correlation = %.3f" % spearmanr(x, y)[0])
pylab.text(0.1, 1.2, "Kendall correlation = %.3f" % kendalltau(x, y)[0])
pylab.title('Scatterplot of sentiment strengths for tweets')
pylab.show()
# pylab.savefig(filebase + 'fnielsen/eps/Nielsen2011New_tweetscatter.eps')

# Ordinary correlation coefficient
C = np.corrcoef(sentiments.transpose())
s1 = corrmatrix2latex(C, columns)
print(s1)

f = open(filebase + '/fnielsen/tex/Nielsen2011New_corrmatrix.tex', 'w')
f.write(s1)
f.close()


# Spearman Rank correlation
C2 = spearmanmatrix(sentiments)
s2 = corrmatrix2latex(C2, columns)
print(s2)

f = open(filebase + '/fnielsen/tex/Nielsen2011New_spearmanmatrix.tex', 'w')
f.write(s2)
f.close()


# Kendall rank correlation
C3 = kendallmatrix(sentiments)
s3 = corrmatrix2latex(C3, columns)
print(s3)

f = open(filebase + '/fnielsen/tex/Nielsen2011New_kendallmatrix.tex', 'w')
f.write(s3)
f.close()
