#!/usr/bin/env python
#
#    The program compares AFINN and ANEW word lists.
#
#    (Copied from Hansen2010Diffusion and extended.)
#
# $Id: Hansen2010Diffusion_anew.py,v 1.3 2010/12/15 15:50:39 fn Exp $

from nltk.stem.wordnet import WordNetLemmatizer
import nltk
import numpy as np
import pylab
import re
from scipy.stats.stats import  kendalltau, spearmanr
import sys
reload(sys)
sys.setdefaultencoding('utf-8')


filebase = '/home/fn/'
filename = filebase + 'fnielsen/data/Nielsen2009Responsible_emotion.csv'
afinn = dict(map(lambda (k,v): (k,int(v)), 
                  [ line.split('\t') for line in open(filename) ]))

filename = filebase + 'data/ANEW.TXT'
anew = dict(map(lambda l: (l[0], float(l[2])) , 
                [ re.split('\s+', line) for line in open(filename).readlines()[41:1075] ]))

lemmatizer = WordNetLemmatizer()
stemmer = nltk.PorterStemmer()

anew_stem = dict([ (stemmer.stem(word), valence) for word, valence in anew.items() ])



def word2anewsentiment_raw(word):
    return anew.get(word, None)


def word2anewsentiment_wordnet(word):
    sentiment = None
    if word in anew:
        sentiment = anew[word]
    else:
        lword = lemmatizer.lemmatize(word)
        if lword in anew:
            sentiment = anew[lword]
        else:
            lword = lemmatizer.lemmatize(word, pos='v')
            if lword in anew:
                sentiment = anew[lword]
    return sentiment


def word2anewsentiment_stem(word):
    return anew_stem.get(stemmer.stem(word), None)



sentiments_raw = []
for word in afinn.keys():
    sentiment_anew = word2anewsentiment_raw(word)
    if sentiment_anew:
        sentiments_raw.append((afinn[word], sentiment_anew))


sentiments_wordnet = []
for word in afinn.keys():
    sentiment_anew = word2anewsentiment_wordnet(word)
    if sentiment_anew:
        sentiments_wordnet.append((afinn[word], sentiment_anew))
        if (afinn[word] > 0 and sentiment_anew < 5) or \
                (afinn[word] < 0 and sentiment_anew > 5):
            print(word)
        

sentiments_stem = []
for word in afinn.keys():
    sentiment_stem_anew = word2anewsentiment_stem(word)
    if sentiment_stem_anew:
        sentiments_stem.append((afinn[word], sentiment_stem_anew))
        if (afinn[word] > 0 and sentiment_stem_anew < 5) or \
                (afinn[word] < 0 and sentiment_stem_anew > 5):
            print(word)


sentiments_raw = np.asarray(sentiments_raw)
pylab.figure(1)
pylab.plot(sentiments_raw[:,0], sentiments_raw[:,1], '.')
pylab.xlabel('Our list')
pylab.ylabel('ANEW')
pylab.title('Correlation between sentiment word lists (Direct match)')
pylab.text(1, 3, "Pearson correlation = %.2f" % np.corrcoef(sentiments_raw.T)[1,0])
pylab.text(1, 2.5, "Spearman correlation = %.2f" % spearmanr(sentiments_raw[:,0], sentiments_raw[:,1])[0])
pylab.text(1, 2, "Kendall correlation = %.2f" % kendalltau(sentiments_raw[:,0], sentiments_raw[:,1])[0])
# pylab.savefig(filebase + 'fnielsen/eps/' + 'Nielsen2011New_anew_raw.eps') 

# pylab.show()

        

sentiments = np.asarray(sentiments_wordnet)
pylab.figure(2)
pylab.plot(sentiments[:,0], sentiments[:,1], '.')
pylab.xlabel('Our list')
pylab.ylabel('ANEW')
pylab.title('Correlation between sentiment word lists (WordNet lemmatizer)')
pylab.text(1, 3, "Pearson correlation = %.2f" % np.corrcoef(sentiments.T)[1,0])
pylab.text(1, 2.5, "Spearman correlation = %.2f" % spearmanr(sentiments[:,0], sentiments[:,1])[0])
pylab.text(1, 2, "Kendall correlation = %.2f" % kendalltau(sentiments[:,0], sentiments[:,1])[0])
# pylab.savefig(filebase + 'fnielsen/eps/' + 'Nielsen2011New_anew_wordnet.eps') 

# pylab.show()



sentiments_stem = np.asarray(sentiments_stem)
pylab.figure(3)
pylab.plot(sentiments_stem[:,0], sentiments_stem[:,1], '.')
pylab.xlabel('My list')
pylab.ylabel('ANEW')
pylab.title('Correlation between sentiment word lists (Porter stemmer)')
pylab.text(1, 3, "Correlation = %.2f" % np.corrcoef(sentiments_stem.T)[1,0])
pylab.text(1, 2.5, "Spearman correlation = %.2f" % spearmanr(sentiments_stem[:,0], sentiments_stem[:,1])[0])
pylab.text(1, 2, "Kendall correlation = %.2f" % kendalltau(sentiments_stem[:,0], sentiments_stem[:,1])[0])
# pylab.savefig(filebase + 'fnielsen/eps/' + 'Nielsen2011New_anew_stem.eps') 

# pylab.show()





