#!/usr/bin/env python
#
# $Id: Nielsen2011New_anewafinn.py,v 1.1 2011/03/13 17:18:10 fn Exp $



import csv
import re
import numpy as np
from nltk.stem.wordnet import WordNetLemmatizer
from nltk import sent_tokenize
import pylab
from scipy.stats.stats import  kendalltau, spearmanr

# This variable determines the data set
mislove = 2

# Filenames
filebase = '/home/fnielsen/'
filename_afinn = filebase + 'fnielsen/data/Nielsen2009Responsible_emotion.csv'
filename_afinn96 = 'AFINN-96.txt'
filename_mislove1 = "results.txt"
filename_mislove2a = "turk.txt"
filename_mislove2b = "turk-results.txt"
filename_anew = filebase + 'data/ANEW.TXT'
filename_gi = filebase + "data/inqtabs.txt"
filename_of = filebase + "data/subjclueslen1-HLTEMNLP05.tff"

# Word splitter pattern
pattern_split = re.compile(r"\W+")


afinn = dict(map(lambda (k,v): (k,int(v)), 
                 [ line.split('\t') for line in open(filename_afinn) ]))


# ANEW
anew = dict(map(lambda l: (l[0], float(l[2]) - 5) , 
                [ re.split('\s+', line) for line in open(filename_anew).readlines()[41:1075] ]))

anewafinn = set(anew.keys()).intersection(afinn.keys())


afinn_intersect = dict([ (k,afinn[k]) for k in anewafinn ])
anew_intersect = dict([ (k, anew[k]) for k in anewafinn ])


# Read Mislove CSV Twitter data: 'tweets' an array of dictionaries
if mislove == 1:
    csv_reader = csv.reader(open(filename_mislove1), delimiter='\t')
    header = []
    tweets = []
    for row in csv_reader:
        if not header:
            header = row
        else:
            tweets.append({'id': row[0], 
                           'quant': int(row[1]), 
                           'score_our': float(row[2]),
                           'score_mean': float(row[3]),
                           'score_std': float(row[4]),
                           'text': row[5], 
                           'scores': map(int, row[6:])})
elif mislove == 2:
    csv_reader = csv.reader(open(filename_mislove2a), delimiter='\t')
    tweets = []
    for row in csv_reader:
        tweets.append({'id': row[2], 
                       'score_mislove2': float(row[1]),
                       'text': row[3]})
    csv_reader = csv.reader(open(filename_mislove2b), delimiter='\t')
    tweets_dict = {}
    header = []
    for row in csv_reader:
        if not header:
            header = row
        else:
            tweets_dict[row[0]] = {'id': row[0], 
                                   'score_mislove': float(row[1]), 
                                   'score_amt_wrong': float(row[2]),
                                   'score_amt': np.mean(map(int, re.split("\s+", " ".join(row[4:]).strip())))
                                   }
    for n in range(len(tweets)):
        tweets[n]['score_mislove'] = tweets_dict[tweets[n]['id']]['score_mislove']
        tweets[n]['score_amt_wrong'] = tweets_dict[tweets[n]['id']]['score_amt_wrong']
        tweets[n]['score_amt'] = tweets_dict[tweets[n]['id']]['score_amt']



# Computer sentiment for each tweet
amt = []
afinn_intersect_sentiment = []
anew_intersect_sentiment = []
for n in range(len(tweets)):
    amt.append(tweets[n]['score_amt'])
    words = pattern_split.split(tweets[n]['text'].lower())
    afinn_sentiments = map(lambda word: afinn_intersect.get(word, 0), words)
    afinn_intersect_sentiment.append(float(sum(afinn_sentiments))/len(afinn_sentiments))
    anew_sentiments = map(lambda word: anew_intersect.get(word, 0), words)
    anew_intersect_sentiment.append(float(sum(anew_sentiments))/len(anew_sentiments))




np.corrcoef(amt, afinn_intersect_sentiment)[0,1]
np.corrcoef(amt, anew_intersect_sentiment)[0,1]

spearmanr(amt, afinn_intersect_sentiment)[0]
spearmanr(amt, anew_intersect_sentiment)[0]
