#!/usr/bin/env python
#
#    Generates a plot of the evolution of the performance as 
#    the word list is extended.
#
# $Id: Nielsen2011New_evolution.py,v 1.2 2011/03/13 23:48:38 fn Exp $


import csv
import re
import numpy as np
import pylab
import random
from scipy import sparse
from scipy.stats.stats import  kendalltau, spearmanr

# This variable determines the data set
mislove = 2

# Filenames
filebase = '/home/fnielsen/'
filename_afinn = filebase + 'fnielsen/data/Nielsen2009Responsible_emotion.csv'
filename_mislove1 = "results.txt"
filename_mislove2a = "turk.txt"
filename_mislove2b = "turk-results.txt"


# Word splitter pattern
pattern_split = re.compile(r"\W+")


afinn = dict(map(lambda (k,v): (k,int(v)), 
                 [ line.split('\t') for line in open(filename_afinn) ]))



# Read Mislove CSV Twitter data: 'tweets' an array of dictionaries
if mislove == 1:
    csv_reader = csv.reader(open(filename_mislove1), delimiter='\t')
    header = []
    tweets = []
    for row in csv_reader:
        if not header:
            header = row
        else:
            tweets.append({'id': row[0], 
                           'quant': int(row[1]), 
                           'score_our': float(row[2]),
                           'score_mean': float(row[3]),
                           'score_std': float(row[4]),
                           'text': row[5], 
                           'scores': map(int, row[6:])})
elif mislove == 2:
    csv_reader = csv.reader(open(filename_mislove2a), delimiter='\t')
    tweets = []
    for row in csv_reader:
        tweets.append({'id': row[2], 
                       'score_mislove2': float(row[1]),
                       'text': row[3]})
    csv_reader = csv.reader(open(filename_mislove2b), delimiter='\t')
    tweets_dict = {}
    header = []
    for row in csv_reader:
        if not header:
            header = row
        else:
            tweets_dict[row[0]] = {'id': row[0], 
                                   'score_mislove': float(row[1]), 
                                   'score_amt_wrong': float(row[2]),
                                   'score_amt': np.mean(map(int, re.split("\s+", " ".join(row[4:]).strip())))
                                   }
    for n in range(len(tweets)):
        tweets[n]['score_mislove'] = tweets_dict[tweets[n]['id']]['score_mislove']
        tweets[n]['score_amt_wrong'] = tweets_dict[tweets[n]['id']]['score_amt_wrong']
        tweets[n]['score_amt'] = tweets_dict[tweets[n]['id']]['score_amt']


allwords = []
for n in range(len(tweets)):
    words = pattern_split.split(tweets[n]['text'].lower())
    tweets[n]['words'] = words
    allwords.extend(words)

print("All words: %d" % len(allwords))
print("Number of unique words in Twitter corpus: %d" % len(set(allwords)))


terms = afinn.keys()
# terms = list(set(allwords).intersection(afinn.keys()))
print("Number of unique words matched to word list: %d" % len(terms))

term2index = dict(zip(terms, range(len(terms))))

M = sparse.lil_matrix((len(tweets), len(terms)))
M = np.zeros((len(tweets), len(terms)))   # Sparse is not necessary
for n in range(len(tweets)):
    for word in tweets[n]['words']:
        if term2index.has_key(word):
            M[n, term2index[word]] = afinn[word]

score_amt = [ t['score_amt'] for t in tweets ]

# Fix resampling seed for reproducibility
random.seed(a=1729)

K = [1, 10, 30, 50, 70, 100, 150, 200, 250, 300, 350, 400, len(terms)]
K = [5, 100, 300, 500, 1000, 1500, 2000, len(terms)]
I = range(len(terms))
resamples = 50
R = np.zeros((resamples, len(K)))
S = np.zeros((resamples, len(K)))
for n in range(len(K)):
    for m in range(resamples):
        J = random.sample(I, K[n])
        score_afinn = M[:,J].sum(axis=1)
        R[m,n] = np.corrcoef(score_amt, score_afinn)[0,1]
        S[m,n] = spearmanr(score_amt, score_afinn)[0]


# pylab.figure(1)
pylab.subplot(2, 1, 1)
pylab.boxplot(R, positions=K, widths=40)
pylab.ylabel('Pearson correlation')
# pylab.xlabel('Word list size')
pylab.title('Evolution of word list performance')
pylab.axis((0, 2600, -0.05, 0.65))
pylab.show()

# pylab.figure(2)
pylab.subplot(2, 1, 2)
pylab.boxplot(S, positions=K, widths=40)
pylab.ylabel('Spearman rank correlation')
pylab.xlabel('Word list size')
# pylab.title('Evolution of word list performance')
pylab.axis((0, 2600, 0.35, 0.6))
pylab.show()

# pylab.savefig(filebase + 'fnielsen/eps/Nielsen2011New_evolution.eps')
